from pydantic import BaseSettings, root_validator

from pyexception_notifier.utils.enums import TargetNotifySources


class Settings(BaseSettings):
    NOTIFY_TO: TargetNotifySources = None
    BOT_TOKEN: str = None
    CHAT_ID: str = None
    SERVICE_NAME: str = None

    class Config:
        env_file = '.env'
        env_prefix = 'EXCEPTION_NOTIFY_'

    @root_validator
    def root_validator_(cls, values):
        if not all((values['BOT_TOKEN'], values['CHAT_ID'], values['SERVICE_NAME'], values['NOTIFY_TO'])):
            raise ValueError(f"please set env vars for exception notifier package {values}")
        return values


settings = Settings()



import requests

from pyexception_notifier.config import settings


def notify_to_telegram(message: str, bot_token: str = None, chat_id: str = None):
    """send message with telegram bot to specific chat"""

    bot_token = settings.BOT_TOKEN if bot_token is None else bot_token
    chat_id = settings.CHAT_ID if chat_id is None else chat_id

    telegram_bot_link = f'https://api.telegram.org/bot{bot_token}/sendMessage?' \
                        f'chat_id={chat_id}&' \
                        f'text={message}&' \
                        f'parse_mode=markdown'
    requests.get(telegram_bot_link)



from pyexception_notifier import init
from pyexception_notifier.exceptions.exceptions import ExampleError
from pyexception_notifier.utils.enums import InstanceTypes


init(instance_type=InstanceTypes.LOCAL.value)


def raise_error():
    raise ExampleError


raise_error()



from pyexception_notifier.config import settings


def parse_common_exception_traceback(exception_traceback: list[str]) -> dict[str, str]:
    traceback_ = '\n'.join(exception_traceback)
    return {
        'traceback': traceback_,
    }


def format_traceback(exception_traceback: list[str]):
    exclamation_mark_emoji = b'\xE2\x9D\x97'.decode('utf-8')

    parsed_traceback = parse_common_exception_traceback(exception_traceback)
    parsed_traceback.update({'service_name': settings.SERVICE_NAME})

    parsed_traceback["traceback"] = parsed_traceback["traceback"].replace('_', r'\_')

    formatted_traceback_message = f'{exclamation_mark_emoji * 3}Exception Notifier{exclamation_mark_emoji * 3}\n\n' \
                                  f'Service: *{parsed_traceback["service_name"]}* \n\n' \
                                  f'{parsed_traceback["traceback"]}'

    return formatted_traceback_message



import sys
from traceback import format_exception

from fastapi import Request

from pyexception_notifier.config import settings
from pyexception_notifier.exceptions.exceptions import UnknownTargetNotifySource, NotifySourceNotGiven, \
    FastApiInstanceNotGiven, UnknownInstanceType
from pyexception_notifier.notify_sources_clients.telegram.client import notify_to_telegram
from pyexception_notifier.utils.enums import TargetNotifySources, InstanceTypes
from pyexception_notifier.utils.exception_parsers import format_traceback


def exception_notify_telegram(exctype, value, tb):
    exception_trace = format_exception(exctype, value, tb)
    parsed_exception_traceback = format_traceback(exception_trace)
    notify_to_telegram(message=parsed_exception_traceback)

    raise exctype(value)


def init(notify_to: TargetNotifySources = None, instance_type: InstanceTypes = 'fastapi', **kwargs) -> None:
    if notify_to is None:
        if settings.NOTIFY_TO is None:
            raise NotifySourceNotGiven

    if instance_type == InstanceTypes.FASTAPI.value:
        app = kwargs.get('app')
        if app is None:
            raise FastApiInstanceNotGiven

        @app.middleware("http")
        async def errors_handling(request: Request, call_next):
            try:
                return await call_next(request)
            except Exception as exc:
                if settings.NOTIFY_TO == TargetNotifySources.TELEGRAM:
                    exception_notify_telegram(*sys.exc_info())
                else:
                    raise UnknownTargetNotifySource
                raise exc

        return

    if instance_type == InstanceTypes.LOCAL.value:
        sys.excepthook = exception_notify_telegram
        return

    raise UnknownInstanceType



class UnknownTargetNotifySource(Exception):
    """Notify source must be one of NotifySources values"""


class NotifySourceNotGiven(Exception):
    """set notify source name as global_exception_handler.init() param or set env var EXCEPTIONS_NOTIFY_TO"""


class ExampleError(Exception):
    """This is an example exception for test exception_notifier package"""

    def __init__(self):
        super().__init__(self.__doc__)


class FastApiInstanceNotGiven(Exception):
    def __init__(self):
        super().__init__('FastApi instance required')


class UnknownInstanceType(Exception):
    """instance type should be one of exception_notifier.utils.enums.InstanceTypes"""



import os

from setuptools import find_packages, setup


if os.getenv('readme') == 'true':
    with open('Readme.md') as f:
        long_description = f.read()
else:
    long_description = ''


setup(
    name='pyexception-notifier',
    packages=find_packages(),
    version='0.0.7',
    description='Exception notifier',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='r.islyamgali',
    license='MIT',
    install_requires=[
        "pydantic",
        "uvicorn",
        "requests(>=2.27,<3.0)",
        "python-dotenv",
        "gunicorn[standard]",
    ],
    python_requires=">=3.8"
)
